import json
import threading
import time
from logging import Handler
from uuid import uuid4

import boto3


class CloudWatchHandler(Handler):
    MAX_BATCH_SIZE = 1048576
    EXTRA_BYTES_PER_EVENT = 26
    REQUESTS_PER_SECOND = 5

    def __init__(self, context, logs_client=None, interval_sec=1.0, retention_days=None):
        super(CloudWatchHandler, self).__init__()
        self.context = context
        self.batch = []
        self.batch_size = 0
        self.interval_sec = max(interval_sec, 1.0 / CloudWatchHandler.REQUESTS_PER_SECOND)
        self.retention_days = retention_days
        self.client = logs_client if logs_client is not None else boto3.client("logs")
        self.log_group = self.context["id"]
        self.sequence_token = None

        self.__create_log_group()
        self.__create_log_stream()

        thread = threading.Thread(target=self.__timed_submit, args=())
        thread.daemon = True
        thread.start()

    def __create_log_group(self):
        exists = False
        paginator = self.client.get_paginator("describe_log_groups")
        for i in paginator.paginate(logGroupNamePrefix=self.log_group):
            if len(i["logGroups"]) > 0:
                exists = True
                break

        if not exists:
            self.client.create_log_group(
                logGroupName=self.log_group,
                tags={
                    "CreatedBy": "logtron",
                    "Created": str(int((time.time() * 1000))),
                },
            )
            if self.retention_days is not None:
                self.client.put_retention_policy(logGroupName=self.log_group, retentionInDays=self.retention_days)

    def __create_log_stream(self):
        self.log_stream = "logtron-{}".format(str(uuid4()))
        self.client.create_log_stream(
            logGroupName=self.log_group,
            logStreamName=self.log_stream,
        )

    def __timed_submit(self):
        while True:
            self.__submit_batch()
            time.sleep(self.interval_sec)

    def __submit_batch(self):
        self.acquire()
        if len(self.batch) == 0:
            self.release()
            return

        args = {
            "logGroupName": self.log_group,
            "logStreamName": self.log_stream,
            "logEvents": self.batch,
        }
        if self.sequence_token is not None:
            args["sequenceToken"] = self.sequence_token

        response = self.client.put_log_events(**args)
        self.sequence_token = response["nextSequenceToken"]

        self.batch = []
        self.batch_size = 0
        self.release()

    def emit(self, record):
        json_obj = json.loads(self.format(record))
        timestamp = json_obj["timestamp"]
        json_obj.pop("timestamp")
        message = json.dumps(json_obj)

        log_entry = {
            "timestamp": timestamp,
            "message": message,
        }

        size = len(json.dumps(log_entry)) + CloudWatchHandler.EXTRA_BYTES_PER_EVENT
        if self.batch_size + size > CloudWatchHandler.MAX_BATCH_SIZE:
            self.__submit_batch()

        self.batch_size += size
        self.batch.append(log_entry)

    def flush(self):
        self.__submit_batch()

    def close(self):
        self.__submit_batch()
        super(CloudWatchHandler, self).close()



import logging
from importlib_metadata import version

from logtron_aws.cloudwatch import CloudWatchHandler
from logtron import autodiscover as autodiscover_base

try:
    __version__ = version(__package__)
except:
    pass


def autodiscover(
    name=None,
    level=logging.INFO,
    config=None,
    refresh=False,
    logs_client=None,
    sts_client=None,
    discover_context=None,
):
    if config is not None:
        config = config.copy()
    else:
        config = {}

    if "handlers" not in config:
        config.update(
            {
                "handlers": ["logtron_aws.cloudwatch.CloudWatchHandler"],
            }
        )

    if logs_client is not None:
        config.update(
            {
                "logtron_aws.cloudwatch.CloudWatchHandler": {"logs_client": logs_client},
            }
        )

    if discover_context is None:
        from logtron_aws.context import discover_context as discover_context_base

        def __discover_context():
            return discover_context_base(sts_client)

        discover_context = __discover_context

    return autodiscover_base(name=name, level=level, config=config, refresh=refresh, discover_context=discover_context)


from logtron_aws.context import discover_context



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['logtron_aws']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1,<2', 'importlib_metadata>=1.7.0,<2.0.0', 'logtron>=0.1.7,<0.2.0']

setup_kwargs = {
    'name': 'logtron-aws',
    'version': '0.1.9',
    'description': 'AWS CloudWatch logging and context auto-discovery for logtron',
    'long_description': '# Logtron-AWS\n\n[![Release](https://img.shields.io/pypi/v/logtron-aws?logo=python&style=flat)](https://pypi.org/project/logtron-aws)\n[![Downloads](https://img.shields.io/pypi/dm/logtron-aws?logo=python&style=flat)](https://pypi.org/project/logtron-aws)\n[![Supported Versions](https://img.shields.io/pypi/pyversions/logtron-aws.svg?logo=python&style=flat)](https://pypi.org/project/logtron-aws)\n[![License](https://img.shields.io/github/license/ilija1/logtron-aws?logo=apache&style=flat)](LICENSE)\n\n[![Build](https://img.shields.io/travis/ilija1/logtron-aws?logo=travis&style=flat)](https://travis-ci.org/ilija1/logtron-aws)\n[![Coverage](https://img.shields.io/codecov/c/gh/ilija1/logtron-aws?logo=codecov&style=flat)](https://codecov.io/gh/ilija1/logtron-aws)\n[![Documentation](https://img.shields.io/readthedocs/logtron-aws?logo=read-the-docs&style=flat)](https://logtron-aws.readthedocs.io/en/latest)\n[![Maintainability](https://img.shields.io/codeclimate/maintainability/ilija1/logtron-aws?logo=code-climate&style=flat)](https://codeclimate.com/github/ilija1/logtron-aws/maintainability)\n[![Tech Debt](https://img.shields.io/codeclimate/tech-debt/ilija1/logtron-aws?logo=code-climate&style=flat)](https://codeclimate.com/github/ilija1/logtron-aws/issues)\n[![Issues](https://img.shields.io/codeclimate/issues/ilija1/logtron-aws?logo=code-climate&style=flat)](https://codeclimate.com/github/ilija1/logtron-aws/issues)\n\n**Logtron-AWS** is a set of AWS-targeted extensions for the [**Logtron**](https://github.com/ilija1/logtron) library.\n\n```python\nimport logtron_aws\nlogger = logtron_aws.autodiscover()\nlogger.info("hello world")\n```\n\nOr\n\n```python\nimport logtron_aws\nlogtron_aws.autodiscover() # Only needs to run once somewhere to configure the root logger\n\nimport logging\nlogger = logging.getLogger()\nlogger.info("hello world")\n```\n\nLogtron-AWS provides a set of extensions for the [Logtron](https://github.com/ilija1/logtron) library to enable features such as:\n\n- Automated log context discovery using AWS STS\n- Log handler for logging directly to CloudWatch Logs\n  - Automatic log group creation\n  - Convention-based log group naming derived from IAM role name\n  - Configurable log retention period\n  - Automated background log batch submission to support high frequency logging\n  - Configureable batch submission time interval\n- Highly configurable if needed, but has sane defaults out-of-the-box\n\n## Installing Logtron-AWS and Supported Versions\n\nLogtron-AWS is available on PyPI:\n\n```shell\n$ python -m pip install logtron-aws\n```\n\nLogtron-AWS officially supports Python 2.7 & 3.5+.\n',
    'author': 'Ilija Stevcev',
    'author_email': 'ilija1@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ilija1/logtron-aws/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*',
}


setup(**setup_kwargs)



import boto3


def discover_context(sts_client=None):
    try:
        sts_client = sts_client if sts_client is not None else boto3.client("sts")
        response = sts_client.get_caller_identity()
        arn = response["Arn"]
        last_part = arn.split(":")[-1]
        parts = last_part.split("/")

        type = parts[0]
        id = parts[1]
        region = sts_client.meta.region_name

        context = {
            "id": id,
            "type": type,
            "region": region,
        }

        if type == "assumed-role" and len(parts) > 2:
            context["session_name"] = parts[2]

        return context
    except Exception as ex:
        print(str(ex))
        return {}
